@@ -4,20 +4,34 @@ use warnings;
 use App::cpm::version;
 use App::cpm::Logger;
 use CPAN::DistnameInfo;
+our $VERSION = '0.210';
+
+use constant STATE_REGISTERED => 0b00001;
+use constant STATE_RESOLVED   => 0b00010; # default
+use constant STATE_FETCHED    => 0b00100;
+use constant STATE_CONFIGURED => 0b01000;
+use constant STATE_INSTALLED  => 0b10000;
 
 sub new {
     my ($class, %option) = @_;
-    bless {_state => 0, %option}, $class;
+    my $uri = delete $option{uri};
+    $uri = [$uri] unless ref $uri;
+    my $distfile = delete $option{distfile};
+    my $source = delete $option{source} || "cpan";
+    my $provides = delete $option{provides} || [];
+    bless {%option, provides => $provides, uri => $uri, distfile => $distfile, source => $source, _state => STATE_RESOLVED}, $class;
 }
 
 for my $attr (qw(
+    source
     configure_requirements
     directory
     distdata
-    distfile
     meta
+    uri
     provides
     requirements
+    ref
 )) {
     no strict 'refs';
     *$attr = sub {
@@ -26,29 +40,48 @@ for my $attr (qw(
         $self->{$attr};
     };
 }
+sub distfile {
+    my $self = shift;
+    $self->{distfile} = shift if @_;
+    $self->{distfile} || $self->{uri}[0];
+}
 
 sub distvname {
     my $self = shift;
     $self->{distvname} ||= do {
-        CPAN::DistnameInfo->new($self->distfile)->distvname || $self->distfile;
+        CPAN::DistnameInfo->new($self->{distfile})->distvname || $self->distfile;
     };
 }
 
-sub append_provide {
+sub overwrite_provide {
     my ($self, $provide) = @_;
-    return if $self->providing($provide->{package});
-    push @{$self->{provides}}, $provide;
+    my $overwrited;
+    for my $exist (@{$self->{provides}}) {
+        if ($exist->{package} eq $provide->{package}) {
+            $exist = $provide;
+            $overwrited++;
+        }
+    }
+    if (!$overwrited) {
+        push @{$self->{provides}}, $provide;
+    }
     return 1;
 }
 
-use constant STATE_RESOLVED   => 0; # default
-use constant STATE_FETCHED    => 1;
-use constant STATE_CONFIGURED => 2;
-use constant STATE_INSTALLED  => 3;
+sub registered {
+    my $self = shift;
+    if (@_ && $_[0]) {
+        $self->{_state} |= STATE_REGISTERED;
+    }
+    $self->{_state} & STATE_REGISTERED;
+}
 
 sub resolved {
     my $self = shift;
-    $self->{_state} == STATE_RESOLVED;
+    if (@_ && $_[0]) {
+        $self->{_state} = STATE_RESOLVED;
+    }
+    $self->{_state} & STATE_RESOLVED;
 }
 
 sub fetched {
@@ -56,7 +89,7 @@ sub fetched {
     if (@_ && $_[0]) {
         $self->{_state} = STATE_FETCHED;
     }
-    $self->{_state} == STATE_FETCHED;
+    $self->{_state} & STATE_FETCHED;
 }
 
 sub configured {
@@ -64,7 +97,7 @@ sub configured {
     if (@_ && $_[0]) {
         $self->{_state} = STATE_CONFIGURED
     }
-    $self->{_state} == STATE_CONFIGURED;
+    $self->{_state} & STATE_CONFIGURED;
 }
 
 sub installed {
@@ -72,14 +105,14 @@ sub installed {
     if (@_ && $_[0]) {
         $self->{_state} = STATE_INSTALLED;
     }
-    $self->{_state} == STATE_INSTALLED;
+    $self->{_state} & STATE_INSTALLED;
 }
 
 sub providing {
     my ($self, $package, $version) = @_;
     for my $provide (@{$self->provides}) {
         if ($provide->{package} eq $package) {
-            if (App::cpm::version->parse($provide->{version})->satisfy($version)) {
+            if (!$version or App::cpm::version->parse($provide->{version})->satisfy($version)) {
                 return 1;
             } else {
                 my $message = sprintf "%s provides %s (%s), but needs %s\n",
