@@ -2,6 +2,7 @@ package App::cpm::Job;
 use strict;
 use warnings;
 use utf8;
+our $VERSION = '0.210';
 
 sub new {
     my ($class, %option) = @_;
@@ -16,7 +17,7 @@ sub _uid {
     my $self = shift;
     my $type = $self->type;
     if (grep { $type eq $_ } qw(fetch configure install)) {
-        "$type " . $self->{distfile};
+        "$type " . $self->distfile;
     } elsif ($type eq "resolve") {
         "$type " . $self->{package};
     } else {
@@ -24,6 +25,11 @@ sub _uid {
     }
 }
 
+sub distfile {
+    my $self = shift;
+    $self->{distfile} || $self->{uri}[0];
+}
+
 sub type {
     my $self = shift;
     $self->{type};
