@@ -5,19 +5,19 @@ use utf8;
 use App::cpm::Distribution;
 use App::cpm::Job;
 use App::cpm::Logger;
-use IO::Handle;
-use IO::Select;
 use Module::CoreList;
 use Module::Metadata;
 use version;
+our $VERSION = '0.210';
 
 sub new {
     my ($class, %option) = @_;
+    if (!exists $Module::CoreList::version{$]}) {
+        die "Module::CoreList does not have your perl $^V entry, abort.\n";
+    }
     bless {
         %option,
-        master => 1,
         installed_distributions => 0,
-        workers => +{},
         jobs => +{},
         distributions => +{},
         _fail_resolve => +{},
@@ -33,120 +33,6 @@ sub fail {
     { resolve => \@fail_resolve, install => \@fail_install };
 }
 
-sub is_master { shift->{master} }
-
-{
-    package
-        App::cpm::_Worker;
-    use JSON::PP;
-    sub new {
-        my ($class, %option) = @_;
-        bless { _written => 0, %option}, $class;
-    }
-    sub has_result { shift->{_written} }
-    sub read_fh  { shift->{read_fh}  }
-    sub write_fh { shift->{write_fh} }
-    sub pid { shift->{pid} }
-    sub write {
-        my ($self, $job) = @_;
-        if ($self->{_written} != 0) { die }
-        $self->{_written}++;
-        $job->in_charge($self->pid);
-        my %copy = %$job;
-        my $encoded = encode_json \%copy;
-        syswrite $self->write_fh, "$encoded\n";
-    }
-    sub work { shift->write(@_) } # alias
-    sub read {
-        my $self = shift;
-        if ($self->{_written} != 1) { die }
-        $self->{_written}--;
-        my $read_fh = $self->read_fh;
-        my $string = <$read_fh>;
-        decode_json $string;
-    }
-    sub result { shift->read } # alias
-}
-
-sub handles {
-    my $self = shift;
-    map { ($_->read_fh, $_->write_fh) } $self->workers;
-}
-
-sub worker {
-    my ($self, $worker_pid) = @_;
-    $self->{workers}{$worker_pid};
-}
-
-sub workers {
-    my $self = shift;
-    values %{$self->{workers}};
-}
-
-sub spawn_worker {
-    my ($self, $cb) = @_;
-    $self->is_master or die;
-    pipe my $read_fh1, my $write_fh1;
-    pipe my $read_fh2, my $write_fh2;
-    my $pid = fork;
-    die "fork failed" unless defined $pid;
-    if ($pid == 0) {
-        $self->{master} = 0;
-        close $_ for $read_fh1, $write_fh2, $self->handles;
-        $write_fh1->autoflush(1);
-        $cb->($read_fh2, $write_fh1);
-        exit;
-    }
-    close $_ for $write_fh1, $read_fh2;
-    $write_fh2->autoflush(1);
-    $self->{workers}{$pid} = App::cpm::_Worker->new(
-        pid => $pid, read_fh => $read_fh1, write_fh => $write_fh2,
-    );
-}
-
-sub _can_read {
-    my ($self, @workers) = @_;
-    $self->is_master or die;
-    @workers = $self->workers unless @workers;
-    my $select = IO::Select->new( map { $_->read_fh } @workers );
-    my @ready = $select->can_read; # blocking
-
-    my @return;
-    for my $worker (@workers) {
-        if (grep { $worker->read_fh == $_ } @ready) {
-            push @return, $worker;
-        }
-    }
-    return @return;
-}
-
-sub shutdown_workers {
-    my $self = shift;
-    close $_ for map { ($_->write_fh, $_->read_fh) } $self->workers;
-    while (%{$self->{workers}}) {
-        my $pid = wait;
-        if ($pid == -1) {
-            warn "wait() returns -1\n";
-        } elsif (my $worker = delete $self->{workers}{$pid}) {
-            close $worker->read_fh;
-        } else {
-            warn "wait() unexpectedly returns $pid\n";
-        }
-    }
-}
-
-sub ready_workers {
-    my ($self, @workers) = @_;
-    $self->is_master or die;
-    @workers = $self->workers unless @workers;
-    my @ready = grep { $_->{_written} == 0 } @workers;
-    return @ready if @ready;
-    $self->_can_read(@workers);
-}
-
-
-## job related method
-
 sub jobs { values %{shift->{jobs}} }
 
 sub add_job {
@@ -162,19 +48,15 @@ sub add_job {
 
 sub get_job {
     my $self = shift;
-    if (my ($job) = grep { !$_->in_charge } $self->jobs) {
-        return $job;
+    if (my @job = grep { !$_->in_charge } $self->jobs) {
+        return @job;
     }
     $self->_calculate_jobs;
     return unless $self->jobs;
-    if (my ($job) = grep { !$_->in_charge } $self->jobs) {
-        return $job;
+    if (my @job = grep { !$_->in_charge } $self->jobs) {
+        return @job;
     }
-
-    my @running_workers = map { $self->worker($_->in_charge) } $self->jobs;
-    my @done_workers = $self->ready_workers(@running_workers);
-    $self->register_result($_->result) for @done_workers;
-    $self->get_job;
+    return;
 }
 
 sub register_result {
@@ -208,22 +90,32 @@ sub _calculate_jobs {
     my @distributions
         = grep { !$self->{_fail_install}{$_->distfile} } $self->distributions;
 
-    if (my @dists = grep { $_->resolved } @distributions) {
+    if (my @dists = grep { $_->resolved && !$_->registered } @distributions) {
         for my $dist (@dists) {
-            $self->add_job(type => "fetch", distfile => $dist->distfile);
+            $dist->registered(1);
+            $self->add_job(
+                type => "fetch",
+                distfile => $dist->{distfile},
+                source => $dist->source,
+                uri => $dist->uri,
+                ref => $dist->ref,
+            );
         }
     }
 
-    if (my @dists = grep { $_->fetched } @distributions) {
+    if (my @dists = grep { $_->fetched && !$_->registered } @distributions) {
         for my $dist (@dists) {
             my ($is_satisfied, @need_resolve)
                 = $self->is_satisfied($dist->configure_requirements);
             if ($is_satisfied) {
+                $dist->registered(1);
                 $self->add_job(
                     type => "configure",
                     meta => $dist->meta,
                     directory => $dist->directory,
-                    distfile => $dist->distfile,
+                    distfile => $dist->{distfile},
+                    source => $dist->source,
+                    uri => $dist->uri,
                 );
             } elsif (@need_resolve) {
                 my $ok = $self->_register_resolve_job(@need_resolve);
@@ -237,17 +129,19 @@ sub _calculate_jobs {
         }
     }
 
-    if (my @dists = grep { $_->configured } @distributions) {
+    if (my @dists = grep { $_->configured && !$_->registered } @distributions) {
         for my $dist (@dists) {
             my ($is_satisfied, @need_resolve)
                 = $self->is_satisfied($dist->requirements);
             if ($is_satisfied) {
+                $dist->registered(1);
                 $self->add_job(
                     type => "install",
                     meta => $dist->meta,
                     distdata => $dist->distdata,
                     directory => $dist->directory,
-                    distfile => $dist->distfile,
+                    distfile => $dist->{distfile},
+                    uri => $dist->uri,
                 );
             } elsif (@need_resolve) {
                 my $ok = $self->_register_resolve_job(@need_resolve);
@@ -266,10 +160,13 @@ sub _register_resolve_job {
     my ($self, @package) = @_;
     my $ok = 1;
     for my $package (@package) {
-        if ($self->{_fail_resolve}{$package->{package}}) {
+        if ($self->{_fail_resolve}{$package->{package}}
+            || $self->{_fail_install}{$package->{package}}
+        ) {
             $ok = 0;
             next;
         }
+
         $self->add_job(
             type => "resolve",
             package => $package->{package},
@@ -334,17 +231,17 @@ sub is_satisfied {
 
         $is_satisfied = 0 if defined $is_satisfied;
         if (!$resolved) {
-            push @need_resolve, { package => $package, version => $version };
+            push @need_resolve, $req;
         }
     }
     return ($is_satisfied, @need_resolve);
 }
 
 sub add_distribution {
-    my ($self, $distribution, $provide) = @_;
+    my ($self, $distribution) = @_;
     my $distfile = $distribution->distfile;
     if (my $already = $self->{distributions}{$distfile}) {
-        $already->append_provide($provide) if $provide;
+        $already->overwrite_provide($_) for @{ $distribution->provides };
         return 0;
     } else {
         $self->{distributions}{$distfile} = $distribution;
@@ -358,7 +255,7 @@ sub _register_resolve_result {
         $self->{_fail_resolve}{$job->{package}}++;
         return;
     }
-    if ($job->{distfile} =~ m{/perl-5[^/]+$}) {
+    if ($job->{distfile} and $job->{distfile} =~ m{/perl-5[^/]+$}) {
         App::cpm::Logger->log(
             result => "FAIL",
             type => "install",
@@ -378,20 +275,27 @@ sub _register_resolve_result {
         return;
     }
 
+    my $provides = $job->{provides};
+    if (!$provides or @$provides == 0) {
+        my $version = App::cpm::version->parse($job->{version}) || 0;
+        $provides = [{package => $job->{package}, version => $version}];
+    }
     my $distribution = App::cpm::Distribution->new(
+        source   => $job->{source},
+        uri      => $job->{uri},
+        provides => $provides,
         distfile => $job->{distfile},
-        provides => [$job->{provide}],
     );
-    $self->add_distribution($distribution, $job->{provide});
+    $self->add_distribution($distribution);
 }
 
 sub _register_fetch_result {
     my ($self, $job) = @_;
     if (!$job->is_success) {
-        $self->{_fail_install}{$job->{distfile}}++;
+        $self->{_fail_install}{$job->distfile}++;
         return;
     }
-    my $distribution = $self->distribution($job->{distfile});
+    my $distribution = $self->distribution($job->distfile);
     $distribution->fetched(1);
     $distribution->configure_requirements($job->{configure_requirements});
     $distribution->directory($job->{directory});
@@ -403,10 +307,10 @@ sub _register_fetch_result {
 sub _register_configure_result {
     my ($self, $job) = @_;
     if (!$job->is_success) {
-        $self->{_fail_install}{$job->{distfile}}++;
+        $self->{_fail_install}{$job->distfile}++;
         return;
     }
-    my $distribution = $self->distribution($job->{distfile});
+    my $distribution = $self->distribution($job->distfile);
     $distribution->configured(1);
     $distribution->distdata($job->{distdata});
     $distribution->requirements($job->{requirements});
@@ -416,10 +320,10 @@ sub _register_configure_result {
 sub _register_install_result {
     my ($self, $job) = @_;
     if (!$job->is_success) {
-        $self->{_fail_install}{$job->{distfile}}++;
+        $self->{_fail_install}{$job->distfile}++;
         return;
     }
-    my $distribution = $self->distribution($job->{distfile});
+    my $distribution = $self->distribution($job->distfile);
     $distribution->installed(1);
     $self->{installed_distributions}++;
     return 1;
